import 'package:flutter/material.dart';

class PickerWidget extends StatefulWidget {
  PickerWidget({
    Key key,
    this.list = const [],
    this.height = 60.0,
    this.labelAlias = 'label',
    this.valueAlias = 'value',
    this.defaultIndex = 0,
    this.onChanged,
  }) : super(key: key);

  final List list;
  final double height;
  final String labelAlias;
  final String valueAlias;
  final int defaultIndex;
  final Function onChanged;

  _PickerWidget createState() => new _PickerWidget();
}

class _PickerWidget extends State<PickerWidget>
    with SingleTickerProviderStateMixin {
  AnimationController _overallMoveController;
  ScrollController _controller;

  int get _length {
    return widget.list.length;
  }

  double get _height {
    return widget.height;
  }

  double get scrollHeight {
    return (_length + 4) * _height;
  }

  @override
  void initState() {
    super.initState();
    double value = widget.defaultIndex * 1.0;
    _overallMoveController = AnimationController(
      vsync: this,
      value: value * _height / scrollHeight,
      duration: const Duration(milliseconds: 200),
    );
    _controller = ScrollController(initialScrollOffset: value * _height);
  }

  @override
  dispose() {
    _overallMoveController.dispose();
    _controller.dispose();
    super.dispose();
  }

  @override
  void didUpdateWidget(covariant PickerWidget oldWidget) {
    if (oldWidget.list.length != widget.list.length ||
        oldWidget.height != widget.height) {
      if (_controller.offset != 0) {
        _controller.jumpTo(0);
      }
    }
    super.didUpdateWidget(oldWidget);
  }

  void _handleDragEnd(double details) {
    int scrollIndex = (details * (_length + 4)).round().clamp(0.0, _length);
    double scroll = scrollIndex * _height;
    if (_controller.offset != scroll) {
      double scrollRatio = (scroll / ((_length + 4) * _height));
      _overallMoveController
          .animateTo(
        scrollRatio,
        curve: Curves.easeIn,
        duration: const Duration(milliseconds: 200),
      )
          .then((value) {
        if (widget.onChanged != null) {
          widget.onChanged(scrollIndex);
        }
        _controller.jumpTo(scroll);
      });
    } else {
      if (widget.onChanged != null) {
        widget.onChanged(scrollIndex);
      }
    }
  }

  @override
  Widget build(BuildContext context) {
    return Container(
      height: _height * 5,
      color: Colors.white,
      child: Stack(
        children: [
          Positioned(
            left: 0,
            right: 0,
            top: 0,
            child: SlideTransition(
              position: Tween<Offset>(
                begin: Offset.zero,
                end: Offset(0.0, -1.0),
              ).animate(_overallMoveController),
              child: FractionallySizedBox(
                widthFactor: 1,
                child: Column(
                  children: [
                    Container(
                      height: _height,
                      color: Colors.transparent,
                      alignment: Alignment.center,
                    ),
                    Container(
                      height: _height,
                      color: Colors.transparent,
                      alignment: Alignment.center,
                    ),
                    ...widget.list
                        .map(
                          (e) => Container(
                            height: _height,
                            alignment: Alignment.center,
                            child: Text(e[widget.labelAlias]),
                          ),
                        )
                        .toList(),
                    Container(
                      height: _height,
                      alignment: Alignment.center,
                    ),
                    Container(
                      height: _height,
                      alignment: Alignment.center,
                    ),
                  ],
                ),
              ),
            ),
          ),
          Container(
            color: Colors.transparent,
            child: Column(
              children: [
                Opacity(
                  opacity: 0.6,
                  child: Container(
                    height: _height,
                    color: Color(0xffffffff),
                  ),
                ),
                Opacity(
                  opacity: 0.4,
                  child: Container(
                    height: _height,
                    color: Color(0xffffffff),
                  ),
                ),
                Container(
                  height: _height - 2,
                  decoration: BoxDecoration(
                    color: Colors.transparent,
                    border: Border(
                      top: BorderSide(
                        color: Color(0xffe3e3e3),
                        width: 1,
                      ),
                      bottom: BorderSide(
                        color: Color(0xffe3e3e3),
                        width: 1,
                      ),
                    ),
                  ),
                ),
                Opacity(
                  opacity: 0.4,
                  child: Container(
                    height: _height,
                    color: Color(0xfff6f6f6),
                  ),
                ),
                Opacity(
                  opacity: 0.6,
                  child: Container(
                    height: _height,
                    color: Color(0xffffffff),
                  ),
                ),
              ],
            ),
          ),
          Container(
            height: _height * 5,
            child: NotificationListener<ScrollNotification>(
              onNotification: (ScrollNotification notification) {
                double progress =
                    notification.metrics.pixels / ((_length + 4) * _height);
                if (notification is ScrollUpdateNotification) {
                  _overallMoveController.value = progress;
                } else if (notification is ScrollEndNotification) {
                  _handleDragEnd(progress);
                }

                return true; //放开此行注释后，进度条将失效
              },
              child: ListView(
                controller: _controller,
                children: List.generate(
                  _length + 4,
                  (index) => Container(
                    color: Colors.transparent,
                    height: _height,
                  ),
                ),
              ),
            ),
          ),
        ],
      ),
    );
  }
}

class MillPicker {
  static year({
    context,
    key,
    Color backgroundColor = const Color(0xffffffff),
    int minYear = 1900,
    int maxYear = 2100,
    initValue,
  }) {
    List list = [];
    for (int i = minYear; i < maxYear; i++) {
      list.add({
        'label': '${i}',
        'value': i,
      });
    }
    int defaultIndex = 0;
    if (initValue != null) {
      defaultIndex = list.indexWhere((el) => el['value'] == initValue);
    }
    Radius r = Radius.circular(20);
    return showModalBottomSheet(
      context: context,
      shape: RoundedRectangleBorder(
        borderRadius: BorderRadius.only(topLeft: r, topRight: r),
      ),
      clipBehavior: Clip.antiAlias,
      backgroundColor: backgroundColor,
      builder: (BuildContext context) {
        return PickerWidget(
          key: key,
          defaultIndex: defaultIndex,
          list: list,
        );
      },
    );
  }

  static month({
    context,
    key,
    Color backgroundColor = const Color(0xffffffff),
    initValue,
  }) {
    List list = [];
    for (int i = 1; i < 13; i++) {
      list.add({
        'label': '${i}',
        'value': i,
      });
    }
    int defaultIndex = 0;
    if (initValue != null) {
      defaultIndex = list.indexWhere((el) => el['value'] == initValue);
    }
    if (defaultIndex < 0) {
      defaultIndex = 0;
    }
    Radius r = Radius.circular(20);
    return showModalBottomSheet(
      context: context,
      shape: RoundedRectangleBorder(
        borderRadius: BorderRadius.only(topLeft: r, topRight: r),
      ),
      clipBehavior: Clip.antiAlias,
      backgroundColor: backgroundColor,
      builder: (BuildContext context) {
        return PickerWidget(
          key: key,
          defaultIndex: defaultIndex,
          list: list,
        );
      },
    );
  }

  static date({
    context,
    key,
    Color backgroundColor = const Color(0xffffffff),
    int minYear = 1900,
    int maxYear = 2100,
    int defaultYear,
    int defaultMonth,
    int defaultDay,
    String title,
    String cancelText,
    String confirmText,
    Function onCancel,
    Function onConfirm,
    TextStyle confirmTextStyle = const TextStyle(
      color: Color(0xff1578ff),
      fontSize: 15,
    ),
    TextStyle cancelTextStyle = const TextStyle(
      color: Color(0xff1578ff),
      fontSize: 15,
    ),
    TextStyle titleTextStyle = const TextStyle(
      color: Color(0xff343434),
      fontSize: 18,
    ),
  }) {
    Radius r = Radius.circular(20);
    String newdate = '';
    return showModalBottomSheet(
      isScrollControlled: true,
      context: context,
      shape: RoundedRectangleBorder(
        borderRadius: BorderRadius.only(topLeft: r, topRight: r),
      ),
      clipBehavior: Clip.antiAlias,
      backgroundColor: backgroundColor,
      builder: (BuildContext context) {
        return FractionallySizedBox(
          widthFactor: 1,
          child: Column(
            mainAxisSize: MainAxisSize.min,
            children: [
              title != null
                  ? Container(
                      height: 60.0,
                      alignment: Alignment.center,
                      child: Row(
                        mainAxisAlignment: MainAxisAlignment.spaceBetween,
                        children: [
                          GestureDetector(
                            onTap: () {
                              if (onCancel != null) {
                                onCancel();
                              }
                              Navigator.pop(context);
                            },
                            child: Container(
                              color: Colors.transparent,
                              padding: EdgeInsets.symmetric(
                                  horizontal: 20, vertical: 10),
                              child: Text(cancelText ?? '取消',
                                  style: cancelTextStyle),
                            ),
                          ),
                          Text(
                            title,
                            style: titleTextStyle,
                          ),
                          GestureDetector(
                            onTap: () {
                              if (onConfirm != null) {
                                onConfirm(newdate);
                              }
                              Navigator.pop(context);
                            },
                            child: Container(
                              color: Colors.transparent,
                              padding: EdgeInsets.symmetric(
                                  horizontal: 20, vertical: 10),
                              child: Text(confirmText ?? '确定',
                                  style: confirmTextStyle),
                            ),
                          )
                        ],
                      ),
                    )
                  : null,
              MillDatePicker(
                minYear: minYear,
                maxYear: maxYear,
                defaultYear: defaultYear,
                defaultMonth: defaultMonth,
                defaultDay: defaultDay,
                onChanged: (date) {
                  newdate = date;
                },
              ),
            ],
          ),
        );
      },
    );
  }
}

class MillDatePicker extends StatefulWidget {
  MillDatePicker({
    this.minYear,
    this.maxYear,
    this.defaultYear,
    this.defaultMonth,
    this.defaultDay,
    this.onChanged,
  });
  final int minYear;
  final int maxYear;
  final int defaultYear;
  final int defaultMonth;
  final int defaultDay;
  final Function onChanged;

  @override
  State<MillDatePicker> createState() => _DatePicker();
}

class _DatePicker extends State<MillDatePicker> {
  List years = [];
  List months = [];
  int _year;
  int _month;
  int _day;

  List mmonth = [1, 3, 5, 7, 8, 10, 12];
  int get maxDay {
    if (_year != null &&
        (_year % 400 == 0 || _year % 100 != 0 && _year % 4 == 0) &&
        _month == 2) {
      return 29;
    }
    if (_month != null) {
      int index = mmonth.indexWhere((el) => el == _month);
      if (index >= 0) {
        return 31;
      }
    }
    if (_month == 2) {
      return 28;
    }
    return 30;
  }

  _change() {
    if (widget.onChanged != null) {
      widget.onChanged('$_year-$_month-$_day');
    }
  }

  @override
  initState() {
    super.initState();
    for (int i = widget.minYear; i < widget.maxYear; i++) {
      years.add({
        'label': '${i}',
        'value': i,
      });
    }
    for (int i = 1; i < 13; i++) {
      months.add({
        'label': '${i}',
        'value': i,
      });
    }

    _year = widget.defaultYear ?? widget.minYear;
    _month = widget.defaultMonth ?? 1;
    _day = widget.defaultDay ?? 1;
  }

  @override
  Widget build(BuildContext context) {
    List days = [];
    for (int i = 1; i < maxDay + 1; i++) {
      days.add({
        'label': '${i}',
        'value': i,
      });
    }
    int defaultYearIndex =
        years.indexWhere((el) => el['value'] == widget.defaultYear);
    int defaultMonthIndex =
        months.indexWhere((el) => el['value'] == widget.defaultMonth);
    int defaultDayIndex =
        days.indexWhere((el) => el['value'] == widget.defaultDay);

    return Row(
      children: [
        Expanded(
          child: PickerWidget(
            key: ValueKey('year'),
            list: years,
            defaultIndex: defaultYearIndex >= 0 ? defaultYearIndex : 0,
            onChanged: (index) {
              setState(() {
                _year = years[index]['value'];
              });
              _change();
            },
          ),
        ),
        Expanded(
          child: PickerWidget(
            key: ValueKey('month'),
            list: months,
            defaultIndex: defaultMonthIndex >= 0 ? defaultMonthIndex : 0,
            onChanged: (index) {
              setState(() {
                _month = months[index]['value'];
                _day = widget.defaultDay ?? 1;
              });
              _change();
            },
          ),
        ),
        Expanded(
          child: PickerWidget(
            key: ValueKey('day'),
            defaultIndex: defaultDayIndex >= 0 ? defaultDayIndex : 0,
            list: days,
            onChanged: (index) {
              _day = days[index]['value'];
              _change();
            },
          ),
        ),
      ],
    );
  }
}
