import 'dart:math';
import 'package:flutter/material.dart';
import 'slide_state.dart';

enum SwipeType {
  stack,
  drawer,
}

abstract class SwipeCard extends StatefulWidget {
  SwipeCard({
    Key key,
    @required this.child,
    this.suffixs = const [],
    this.prefixs = const [],
    this.preRatio = 0.5,
    this.sufRatio = 0.5,
    this.type = SwipeType.stack,
  }) : super(key: key);

  final Widget child;
  final List<Widget> suffixs;
  final List<Widget> prefixs;
  final double preRatio;
  final double sufRatio;
  final SwipeType type;

  Widget buildFix(double actionValue, List<Widget> children,
      AnimationController _overallMoveController, double ratio,
      [CurvedAnimation _actionsMoveAnimation]);
  _MillSwipeCard createState() => _MillSwipeCard();
}

class _MillSwipeCard extends State<SwipeCard>
    with SingleTickerProviderStateMixin {
  AnimationController _overallMoveController;
  CurvedAnimation _actionsMoveAnimation;

  double actionValue = 0.0;
  double _dragExtent = 0.0;
  double _displacement = 0.0;
  AnimationStatus animationStatus;

  SlideState state = SlideState.dismissed;

  double get _overallDragAxisExtent {
    final Size size = context.size;
    return size.width;
  }

  double get ratio {
    return actionValue > 0 ? widget.preRatio : widget.sufRatio;
  }

  @override
  void initState() {
    super.initState();
    _overallMoveController = AnimationController(
      vsync: this,
      duration: const Duration(milliseconds: 200),
    )
      ..addStatusListener(_handleDismissStatusChanged)
      ..addListener(_handleOverallPositionChanged);
    if (widget.type == SwipeType.drawer) {
      _actionsMoveAnimation = CurvedAnimation(
        parent: _overallMoveController,
        curve: Interval(0.0, ratio),
      );
    }
  }

  @override
  void dispose() {
    _overallMoveController.dispose();
    super.dispose();
  }

  _handleDismissStatusChanged(AnimationStatus status) {
    animationStatus = status;
  }

  _handleOverallPositionChanged() {
    if (state != SlideState.dismissed &&
        _overallMoveController.value <= 0.02 &&
        _overallMoveController.value > 0) {
      close();
    }
  }

  _handleDragStart(DragStartDetails details) {
    _displacement = 0.0;
  }

  void _handleDragUpdate(DragUpdateDetails details) {
    final double delta = details.primaryDelta;
    _dragExtent += delta;
    _displacement += delta;
    double maxcamp = state == SlideState.dismissed ? ratio : ratio + 0.08;
    setState(() {
      if (widget.suffixs.length > 0) {
        actionValue = _dragExtent >= 0 ? 1 : -1;
      }
      if (_dragExtent >= 0) {
        actionValue = widget.prefixs.length > 0 ? 1 : 0;
      } else {
        actionValue = widget.suffixs.length > 0 ? -1 : 0;
      }
      if (actionValue != 0) {
        _overallMoveController.value =
            (_dragExtent.abs() / _overallDragAxisExtent).clamp(0.0, maxcamp);
      }
    });
  }

  void _handleDragEnd(DragEndDetails details) {
    if (state != SlideState.dismissed) {
      if (_displacement.abs() > min(26, _overallDragAxisExtent * ratio * 0.2)) {
        close();
      } else {
        open();
      }
    } else {
      if (_displacement.abs() > min(26, _overallDragAxisExtent * ratio * 0.2) &&
          actionValue != 0) {
        open();
      } else {
        close();
      }
    }
  }

  void close() {
    _dragExtent = 0;
    state = SlideState.dismissed;
    if (actionValue != 0) {
      setState(() {
        _overallMoveController
            .animateTo(
          0,
          curve: Curves.easeIn,
          duration: const Duration(milliseconds: 150),
        )
            .then((value) {
          actionValue = 0;
        });
      });
    }
  }

  void open() {
    _overallMoveController.animateTo(
      ratio,
      curve: Curves.easeIn,
      duration: const Duration(milliseconds: 200),
    );
    if (actionValue > 0) {
      state = SlideState.right;
      _dragExtent = _overallDragAxisExtent * ratio;
    } else if (actionValue < 0) {
      state = SlideState.left;
      _dragExtent = -_overallDragAxisExtent * ratio;
    }
  }

  Widget build(BuildContext context) {
    List<Widget> children = actionValue > 0 ? widget.prefixs : widget.suffixs;
    Widget fix = widget.buildFix(
      actionValue,
      children,
      _overallMoveController,
      ratio,
      _actionsMoveAnimation,
    );

    return GestureDetector(
      onHorizontalDragStart: _handleDragStart,
      onHorizontalDragUpdate: _handleDragUpdate,
      onHorizontalDragEnd: _handleDragEnd,
      child: Stack(
        children: [
          fix,
          SlideTransition(
            position: Tween<Offset>(
              begin: Offset.zero,
              end: Offset(actionValue, 0.0),
            ).animate(_overallMoveController),
            child: FractionallySizedBox(
              alignment: Alignment.center,
              widthFactor: 1,
              child: Container(
                color: Colors.white,
                child: widget.child,
              ),
            ),
          ),
        ],
      ),
    );
  }
}

class MillSwipeCard extends SwipeCard {
  MillSwipeCard({
    Key key,
    child,
    suffixs = const <Widget>[],
    prefixs = const <Widget>[],
    preRatio = 0.5,
    sufRatio = 0.5,
  }) : super(
          key: key,
          child: child,
          suffixs: suffixs,
          prefixs: prefixs,
          preRatio: preRatio,
          sufRatio: sufRatio,
          type: SwipeType.stack,
        );

  @override
  buildFix(actionValue, List<Widget> children, _overallMoveController, ratio,
      [_actionsMoveAnimation]) {
    return Positioned.fill(
      child: FractionallySizedBox(
        alignment: Alignment(
          -actionValue,
          0.0,
        ),
        widthFactor:
            children.length > 0 ? max(ratio, _overallMoveController.value) : 0,
        heightFactor: null,
        child: Flex(
          direction: Axis.horizontal,
          children: children
              .map(
                (el) => Expanded(
                  child: el,
                ),
              )
              .toList(),
        ),
      ),
    );
  }
}

class MillSwipeDrawer extends SwipeCard {
  MillSwipeDrawer({
    Key key,
    child,
    suffixs = const <Widget>[],
    prefixs = const <Widget>[],
    preRatio = 0.5,
    sufRatio = 0.5,
  }) : super(
          key: key,
          child: child,
          suffixs: suffixs,
          prefixs: prefixs,
          preRatio: preRatio,
          sufRatio: sufRatio,
          type: SwipeType.drawer,
        );

  @override
  Widget buildFix(
      actionValue, List<Widget> children, _overallMoveController, ratio,
      [_actionsMoveAnimation]) {
    final startOffset = Offset(-actionValue, 0.0);
    final animations = Iterable.generate(children.length).map((index) {
      return Tween<Offset>(
        begin: startOffset,
        end: startOffset * (index - children.length + 1.0),
      ).animate(_actionsMoveAnimation);
    }).toList();
    return Positioned.fill(
      child: Stack(
        alignment: actionValue > 0 ? Alignment.topLeft : Alignment.topRight,
        children: List.generate(
          children.length,
          (index) {
            return SlideTransition(
              position: animations[index],
              child: FractionallySizedBox(
                alignment: Alignment(
                  -actionValue,
                  0.0,
                ),
                widthFactor: children.length > 0
                    ? max<double>(ratio, _overallMoveController.value) /
                        children.length
                    : 0,
                heightFactor: null,
                child: children[children.length - index - 1],
              ),
            );
          },
        ),
      ),
    );
  }
}
